/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import "../Applications/applications.js" as ApplicationsModel

QtObject {
    id: applicationManager

    /* This file fakes the abilities of the ApplicationManagerWrapper for platforms where
       Ubuntu.Application is not available. */
    property ListModel mainStageApplications: ListModel {}
    property ListModel sideStageApplications: ListModel {}
    property var mainStageFocusedApplication
    property var sideStageFocusedApplication
    property bool sideStageEnabled: true
    signal focusRequested(string desktopFile) //unused as not functional on desktop
    property bool keyboardVisible: false
    property int keyboardHeight: 0

    function activateApplication(desktopFile, argument) {
        var app = ApplicationsModel.createApplicationForDesktopFile(desktopFile);
        if(!app) {
            console.log("ERROR: Application '" + desktopFile + "' not found, unable to activate");
            return false;
        }

        if (!app.exec || !app.desktopFile) {
            console.log("ERROR: Application '" + desktopFile + "' has no exec or desktop file");
            return false;
        }

        app.stage = applicationManager.sideStageEnabled ? app.stage : ApplicationsModel.MainStage
        var application;
        application = getApplicationFromDesktopFile(app.desktopFile, app.stage);
        if (application != null) {
            return application;
        }

        // construct application model, adding info to app needed for ApplicationWindowFake
        application = {
            'icon': app.icon,
            'handle': app.desktopFile,
            'name': app.name,
            'fullscreen': app.fullscreen,
            'desktopFile': app.desktopFile,
            'stage': app.stage,
            'argument': argument,
            'window': null
        };

        // Create window for the application
        var component = Qt.createComponent("ApplicationWindowFake.qml");
        if (component.status === Component.Error) {
            console.log("ERROR: ApplicationWindowFake.qml failed to load", component.errorString());
            return null;
        }

        var sideStage = applicationManager.sideStageEnabled
                        && (application.stage == ApplicationsModel.SideStage);
        // attach window and add to front of running applications model
        __lastUsedLayer += __focusIncrement;
        application.window = component.createObject(fakeWindowContainer, {"application": application,
                                                    "anchors.right": fakeWindowContainer.right,
                                                    "y": (app.fullscreen) ? 0 : shell.panelHeight,
                                                    "width": sideStage ? fakeWindowContainer.sideStageWidth : fakeWindowContainer.width,
                                                    "height": fakeWindowContainer.height,
                                                    "z": __lastUsedLayer
                                                });
        var applications = sideStage ? applicationManager.sideStageApplications
                                     : applicationManager.mainStageApplications;
        applications.append(application);
        return application;
    }

    property int __lastUsedLayer: 0
    property int __focusIncrement: 10

    function focusApplication(application) {
        if (application == null) {
            return;
        }

        var sideStage = (application.stage == ApplicationsModel.SideStage
                         && applicationManager.sideStageEnabled);
        var applications = sideStage ? applicationManager.sideStageApplications
                                     : applicationManager.mainStageApplications;


        __lastUsedLayer += __focusIncrement;
        application.window.z = __lastUsedLayer;

        if (sideStage) {
            sideStageFocusedApplication = application;
        } else {
            mainStageFocusedApplication = application;
        }
    }

    function unfocusCurrentApplication() {
        sideStageFocusedApplication = null;
        mainStageFocusedApplication = null;
    }

    function moveRunningApplicationStackPosition(from, to, stage) {
        var sideStage = (stage == ApplicationsModel.SideStage);
        var applications = sideStage ? applicationManager.sideStageApplications
                                     : applicationManager.mainStageApplications;

        if (from !== to && applications.count > 0 && from >= 0 && to >= 0) {
            applications.move(from, to, 1);
        }
    }

    function getApplicationFromDesktopFile(desktopFile, stage) {
        var sideStage = (stage == ApplicationsModel.SideStage);
        var applications = sideStage ? applicationManager.sideStageApplications
                                     : applicationManager.mainStageApplications;

        for (var i = 0; i < applications.count; i++ ) {
            var application = applications.get(i);
            if (application.desktopFile == desktopFile) {
                return application;
            }
        }
        return null;
    }
}
