/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 0.1

Item {
    id: root

    property alias model: userList.model
    property alias currentIndex: userList.currentIndex

    readonly property int cellHeight: units.gu(6)
    readonly property int highlightedHeight: units.gu(10)
    readonly property int moveDuration: 200
    readonly property bool guestAccount: userList.model.get(userList.currentIndex).password === undefined

    signal selected(int uid)
    signal unlocked(int uid)

    Rectangle {
        id: highlightItem
        anchors {
            left: parent.left
            right: parent.right
            verticalCenter: parent.verticalCenter
        }
        height: root.highlightedHeight
        color: Qt.rgba(0, 0, 0, 0)
        border.color: Qt.rgba(1, 1, 1, 0.1)
        border.width: units.dp(1)
        radius: units.gu(1)
    }

    PathView {
        id: userList
        objectName: "userList"

        anchors.fill: parent

        preferredHighlightBegin: count == 2 ? 0.3 : 0.5
        preferredHighlightEnd: count == 2 ? 0.3 : 0.5
        highlightRangeMode: PathView.StrictlyEnforceRange
        highlightMoveDuration: root.moveDuration
        pathItemCount: 5
        flickDeceleration: 10000
        interactive: count > 2

        readonly property bool movingInternally: moveTimer.running || userList.moving
        readonly property int pathHeight: root.highlightedHeight +  (root.cellHeight * Math.floor(Math.min(count, pathItemCount) / 2)) * 2
        readonly property int pathStartMargin: (userList.height - pathHeight) / 2
        readonly property int offsetStartDistance: units.gu(7)
        readonly property int fadeOutDistance: pathStartMargin - units.gu(1)

        onCurrentIndexChanged: {
            passwordInput.text = "";
            passwordInput.focus = false
        }

        onMovingInternallyChanged: {
            // Only emit the selected signal once we stop moving to avoid frequent background changes
            if (!movingInternally) {
                root.selected(userList.currentIndex);
            }
        }

        path: count == 2 ? pathForTwo : pathForMore

        Path {
            id: pathForTwo
            startX: userList.width / 2
            startY: userList.pathStartMargin
            PathAttribute { name: "itemOpacity"; value: 1 }
            PathAttribute { name: "itemOffset"; value: units.gu(2) }

            PathLine { x: userList.width / 2; y: userList.height - userList.pathStartMargin}
            PathAttribute { name: "itemOpacity"; value: 1 }
            PathAttribute { name: "itemOffset"; value: units.gu(2) }
        }

        Path {
            id: pathForMore
            startX: userList.width / 2
            startY: 0
            PathAttribute { name: "itemOpacity"; value: 0 }
            PathAttribute { name: "itemOffset"; value: 0 }

            PathLine { x: userList.width / 2; y: userList.fadeOutDistance }
            PathPercent { value: 0.09}
            PathAttribute { name: "itemOpacity"; value: 0 }
            PathAttribute { name: "itemOffset"; value: 0 }

            PathLine { x: userList.width / 2; y: userList.pathStartMargin }
            PathPercent { value: 0.1}
            PathAttribute { name: "itemOpacity"; value: 0.2 }
            PathAttribute { name: "itemOffset"; value: 0 }

            PathLine { x: userList.width / 2; y: userList.height / 2 - userList.offsetStartDistance }
            PathPercent { value: 0.4}
            PathAttribute { name: "itemOpacity"; value: 1 }
            PathAttribute { name: "itemOffset"; value: 0 }

            PathLine { x: userList.width / 2; y: userList.height / 2 }
            PathPercent { value: 0.5}
            PathAttribute { name: "itemOpacity"; value: 1 }
            PathAttribute { name: "itemOffset"; value: -units.gu(3) }

            PathLine { x: userList.width / 2; y: userList.height / 2 + userList.offsetStartDistance }
            PathPercent { value: 0.6}
            PathAttribute { name: "itemOpacity"; value: 1 }
            PathAttribute { name: "itemOffset"; value: 0 }

            PathLine { x: userList.width / 2; y: userList.height - userList.pathStartMargin }
            PathPercent { value: 0.9}
            PathAttribute { name: "itemOpacity"; value: 0.2 }
            PathAttribute { name: "itemOffset"; value: 0 }

            PathLine { x: userList.width / 2; y: userList.height - userList.fadeOutDistance }
            PathPercent { value: 0.91}
            PathAttribute { name: "itemOpacity"; value: 0 }
            PathAttribute { name: "itemOffset"; value: 0 }

            PathLine { x: userList.width / 2; y: userList.height }
            PathPercent { value: 1}
            PathAttribute { name: "itemOpacity"; value: 0 }
            PathAttribute { name: "itemOffset"; value: 0 }
        }

        delegate: Item {
            objectName: "username" + index
            width: parent.width
            // Adding 2 gus in height to close gaps in draggable area between items
            height: (index == userList.currentIndex ? root.highlightedHeight : root.cellHeight) + units.gu(2)
            opacity: PathView.itemOpacity
            property int itemOffset: PathView.itemOffset

            Label {
                anchors {
                    left: parent.left
                    leftMargin: units.gu(2)
                    verticalCenter: parent.verticalCenter
                    verticalCenterOffset: parent.itemOffset
                    right: parent.right
                    rightMargin: units.gu(2)
                }
                text: name
                color: "white"
                elide: Text.ElideRight
            }

            MouseArea {
                anchors.fill: parent
                enabled: userList.currentIndex !== index
                onClicked: {
                    moveTimer.start();

                    // In case we have only 2 items, setting current index to
                    // an absolute value, scrolls the path to the wrong direction.
                    // Lets use incrementCurrentIndex() to explicitly specify the
                    // direction.
                    if (userList.count == 2) {
                        userList.incrementCurrentIndex();
                    } else {
                        userList.currentIndex = index;
                    }
                }
            }
        }

        // This is needed because PathView.moving is not true if the PathView 
        // moves because of an internal event (e.g. currentIndex has changed)
        Timer {
            id: moveTimer
            running: false
            repeat: false
            interval: root.moveDuration
        }
    }

    TextField {
        id: passwordInput
        objectName: "passwordInput"
        anchors {
            bottom: highlightItem.bottom
            horizontalCenter: parent.horizontalCenter
            margins: units.gu(1)
        }
        height: units.gu(4.5)
        width: parent.width - anchors.margins * 2
        placeholderText: root.guestAccount ? "Tap to unlock" : "Password"
        echoMode: TextInput.Password
        opacity: userList.movingInternally ? 0 : 1

        Behavior on opacity {
            NumberAnimation { duration: 100 }
        }

        onAccepted: {
            if (text === userList.model.get(userList.currentIndex).password) {
                root.unlocked(userList.currentIndex);
                passwordInput.focus = false;
            } else {
                wrongPasswordAnimation.start();
            }
            text = "";
        }

        Image {
            anchors {
                right: parent.right
                rightMargin: units.gu(2)
                verticalCenter: parent.verticalCenter
            }
            visible: root.guestAccount
            source: "graphics/icon_arrow.png"
        }

        SequentialAnimation {
            id: wrongPasswordAnimation
            NumberAnimation {
                target: passwordInput
                property: "anchors.horizontalCenterOffset"
                duration: 50
                easing.type: Easing.InQuad
                to: units.gu(2)
            }
            NumberAnimation {
                target: passwordInput
                property: "anchors.horizontalCenterOffset"
                duration: 100
                easing.type: Easing.InOutQuad
                to: -units.gu(2)
            }
            NumberAnimation {
                target: passwordInput
                easing.type: Easing.OutElastic
                properties: "anchors.horizontalCenterOffset"
                to: 0
                duration: 400
                easing.overshoot: units.gu(2)
            }
        }

        Connections {
            target: Qt.inputMethod
            onVisibleChanged: {
                if (!Qt.inputMethod.visible) {
                    passwordInput.focus = false;
                }
            }
        }

    }
    MouseArea {
        anchors.fill: passwordInput
        enabled: root.guestAccount
        onClicked: {
            root.unlocked(userList.currentIndex);
        }
    }
}
