/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.ChewieUI 0.1 as ChewieUI

import "../Components"
import "../Components/Math.js" as MathLocal

Showable {
    id: indicators

    property int progress: 0
    property int openedHeight: units.gu(71)
    property int panelHeight: units.gu(3)
    property bool pinnedMode: true  //should be set true if indicators menu can cover whole screen

    property int hintValue
    readonly property int lockThreshold: shell.height/2
    property bool fullyOpened: false
    property bool partiallyOpened: false

    property Revealer revealer: null

    height: menuContent.height + handle.height + menuContent.y

    onProgressChanged: {
        var contentProgress = progress - handle.height
        if (!showAnimation.running && !hideAnimation.running && !revealer.hintingAnimation.running) {
            if (contentProgress <= hintValue && indicators.state == "reveal") {
                indicators.state = "hint"
                menuContent.hideAll()
            } else if (contentProgress > hintValue && contentProgress < lockThreshold) {
                menuContent.showMenu()
                indicators.state = "reveal"
            } else if (contentProgress >= lockThreshold && lockThreshold > 0) {
                indicators.state = "locked"
            }
        }

        if (contentProgress == 0) {
            menuContent.releaseContent()
        }
    }

    state: "initial"

    function handlePress() {
        menuContent.hideAll()
        menuContent.activateContent()
        indicators.state = "hint"
    }

    function openOverview() {
        indicatorRow.currentItem = null
        menuContent.showOverview()
    }

    function calculateCurrentItem(xValue, useBuffer) {
        var rowCoordinates
        var itemCoordinates
        var currentItem
        var distanceFromRightEdge
        var bufferExceeded = false

        /*
          If user drags the indicator handle bar down a distance hintValue or less, this is 0.
          If bar is dragged down a distance greater than or equal to lockThreshold, this is 1.
          Otherwise it contains the bar's location as a fraction of the distance between hintValue (is 0) and lockThreshold (is 1).
        */
        var verticalProgress = MathLocal.clamp((indicators.progress - handle.height - hintValue) / (lockThreshold - hintValue), 0, 1)

        /*
          Percentage of an indicator icon's width the user's press can stray horizontally from the
          focused icon before we change focus to another icon. E.g. a value of 0.5 means you must
          go right a distance of half an icon's width before focus moves to the icon on the right
        */
        var maxBufferThreshold = 0.5

        /*
          To help users find the indicator of their choice while opening the indicators, we add logic to add a
          left/right buffer to each icon so it is harder for the focus to be moved accidentally to another icon,
          as the user moves their finger down, but yet allows them to switch indicator if they want.
          This buffer is wider the further the user's finger is from the top of the screen.
        */
        var effectiveBufferThreshold = maxBufferThreshold * verticalProgress;

        if (indicators.state == "commit" || indicators.state == "locked" || showAnimation.running || hideAnimation.running) return

        rowCoordinates = indicatorRow.mapToItem(indicatorRow.row, xValue, 0);
        // get the current delegate
        currentItem = indicatorRow.row.childAt(rowCoordinates.x, 0);
        if (currentItem && currentItem != indicatorRow.currentItem ) {
            itemCoordinates = indicatorRow.row.mapToItem(currentItem, rowCoordinates.x, 0);
            distanceFromRightEdge = (currentItem.width - itemCoordinates.x) / (currentItem.width)
            if (Math.abs(currentItem.ownIndex - indicatorRow.currentItemIndex) > 1) {
                bufferExceeded = true
            } else {
                if (indicatorRow.currentItemIndex < currentItem.ownIndex && distanceFromRightEdge < (1 - effectiveBufferThreshold)) {
                    bufferExceeded = true
                } else if (indicatorRow.currentItemIndex > currentItem.ownIndex && distanceFromRightEdge > effectiveBufferThreshold) {
                    bufferExceeded = true
                }
            }
            if ((!useBuffer || (useBuffer && bufferExceeded)) || indicatorRow.currentItem < 0 || indicatorRow.currentItem == null)  {
                indicatorRow.currentItem = currentItem;
            }
        }
    }
    // eater
    MouseArea {
        anchors {
            top: parent.top
            bottom: handle.bottom
            left: parent.left
            right: parent.right
        }
    }

    MenuContent {
        id: menuContent

        anchors {
            left: parent.left
            right: parent.right
            top: indicatorRow.bottom
        }
        height: progress - y
        indicatorsModel: indicatorsModel
        animate: false
        clip: indicators.partiallyOpened

        onMenuSelected: {
            indicatorRow.setItem(index)
        }

        //small shadow gradient at bottom of menu
        Rectangle {
            anchors {
                left: parent.left
                right: parent.right
                bottom: parent.bottom
            }
            height: units.gu(0.5)
            gradient: Gradient {
                GradientStop { position: 0.0; color: "transparent" }
                GradientStop { position: 1.0; color: "black" }
            }
            opacity: 0.4
        }
    }

    Rectangle {
        id: handle

        color:  menuContent.backgroundColor

        anchors {
            left: parent.left
            right: parent.right
            top: menuContent.bottom
        }
        height: Math.min(handleImage.height, progress - handleImage.height)
        clip: height < handleImage.height

        BorderImage {
            id: handleImage
            source: "graphics/handle.sci"
            height: panelHeight
            anchors {
                left: parent.left
                right: parent.right
                bottom: parent.bottom
            }
        }
        MouseArea { //prevent clicks passing through
            anchors.fill: parent
        }
    }

    PanelBackground {
        anchors.fill: indicatorRow
    }

    IndicatorRow {
        id: indicatorRow
        anchors {
            left: parent.left
            right: parent.right
        }
        height: indicators.panelHeight
        indicatorsModel: indicatorsModel
        state: indicators.state
        overviewActive: menuContent.overviewActive

        onCurrentItemIndexChanged: menuContent.currentIndex = currentItemIndex

    }

    ChewieUI.PluginModel {
        id: indicatorsModel
        Component.onCompleted: load()
    }

    Connections {
        target: hideAnimation
        onRunningChanged: {
            if (hideAnimation.running) {
                indicators.state = "initial"
                menuContent.hideAll()
            } else  {
                if (state == "initial") indicatorRow.setDefaultItem()
            }
        }
    }
    Connections {
        target: showAnimation
        onRunningChanged: {
            if (showAnimation.running) {
                if (indicators.state == "initial") {
                    openOverview()
                } 
                indicators.state = "commit"
            }
        }
    }

    Connections {
        target: revealer
        onLateralPositionChanged: {
            var buffer = revealer.dragging ? true : false
            indicators.calculateCurrentItem(revealer.lateralPosition, buffer)
        }
    }

    //FIXME: when 'states' is changed, "state" is reset to the empty string. Need to find
    //a way to detect this and restore the correct state again.
    states: (pinnedMode) ? pinnedModeStates : offScreenModeStates

    property list<State> offScreenModeStates: [
        State {
            name: "initial"
            PropertyChanges { target: menuContent;  y: panelHeight }
        },
        State {
            name: "hint"
            PropertyChanges { target: indicatorRow; y: panelHeight }
            PropertyChanges { target: menuContent;  y: 2*panelHeight }
        },
        State {
            name: "reveal"
            extend: "hint"
            PropertyChanges { target: menuContent; animate: true }
            StateChangeScript { script: calculateCurrentItem(revealer.lateralPosition, false); }
        },
        State {
            name: "locked"
            extend: "hint"
        },
        State {
            name: "commit"
            extend: "hint"
        }
    ]

    property list<State> pinnedModeStates: [
        State {
            name: "initial"
            PropertyChanges { target: menuContent; y: panelHeight }
        },
        State {
            name: "hint"
        },
        State {
            name: "reveal"
            PropertyChanges { target: menuContent; animate: true }
            StateChangeScript { script: calculateCurrentItem(revealer.lateralPosition, false); }
        },
        State {
            name: "locked"
        },
        State {
            name: "commit"
        }
    ]

    transitions: [
        Transition  {
            NumberAnimation {targets: [indicatorRow, menuContent]; property: "y"; duration: 300; easing.type: Easing.OutCubic}
        }
    ]
}
