//
// Copyright © 2013 Canonical Ltd.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 3 as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef PLUGINS_NOTIFICATIONS_NOTIFICATION_H
#define PLUGINS_NOTIFICATIONS_NOTIFICATION_H

#include <QString>
#include <QStringList>
#include <QImage>
#include <QQmlProperty>
#include <QQmlListProperty>

#include "action.h"

namespace unity
{
namespace notifications
{
enum class Urgency
{
	low,
	normal,
	critical
};

enum class Type
{
	confirmation,
	ephemeral,
	interactive,
	snap_decision,
	none
};

enum Hint // there's no "append" hint - it should be handled in NotificationSource
{
	button_tint = 1 << 0,
	synchronous = 1 << 1,
	icon_only = 1 << 2
};

class Action;

class Notification : public QObject
{
	Q_OBJECT

	public:
		typedef int Id;

		Q_ENUMS(Type);
		Q_ENUMS(Urgency);

		Q_DECLARE_FLAGS(Hints, Hint);

		Q_PROPERTY(Type type READ type NOTIFY typeChanged);
		Q_PROPERTY(Urgency urgency READ urgency NOTIFY urgencyChanged);
		Q_PROPERTY(Id id READ id);
		Q_PROPERTY(QString summary READ summary NOTIFY summaryChanged);
		Q_PROPERTY(QString body READ body NOTIFY bodyChanged);
		Q_PROPERTY(int value READ value NOTIFY valueChanged);
		Q_PROPERTY(QImage icon READ icon NOTIFY iconChanged);
		Q_PROPERTY(QImage secondaryIcon READ secondaryIcon NOTIFY secondaryIconChanged);
		Q_PROPERTY(QQmlListProperty<Action> actions READ actionsQmlList NOTIFY actionsChanged);
		Q_PROPERTY(Hints hints READ hints NOTIFY hintsChanged);

		Notification (QObject* = 0) {}
		virtual ~Notification ();

		Notification(const Notification& rhs) = delete;
		Notification& operator=(const Notification& rhs) = delete;

		virtual Type type() const;
		virtual Urgency urgency() const;
		virtual Id id() const;
		virtual QString summary() const;
		virtual QString body() const;
		virtual int value() const;
		virtual const QImage& icon() const;
		virtual const QImage& secondaryIcon() const;
		virtual const QVector<QSharedPointer<Action>>& actions() const;
		virtual const QQmlListProperty<Action>& actionsQmlList() const;
		virtual Hints hints() const;

		virtual void setType(Type type);
		virtual void setUrgency(Urgency urgency);
		virtual void setSummary(const QString& summary);
		virtual void setBody(const QString& body);
		virtual void setValue(const int value);
		virtual void setIcon(const QImage& icon);
		virtual void setSecondaryIcon(const QImage& secondaryIcon);
		virtual void setActions(const QVector<QSharedPointer<Action>>& actions);
		virtual void setHints(Hints hints);

		Q_SLOT virtual void onHovered(bool hovered); // stop/start timeout timer
		Q_SLOT virtual void onDisplayed(bool displayed); // start/restart the timer

		Q_SIGNAL void typeChanged(Type);
		Q_SIGNAL void urgencyChanged(Urgency);
		Q_SIGNAL void summaryChanged(const QString&);
		Q_SIGNAL void bodyChanged(const QString&);
		Q_SIGNAL void valueChanged(int);
		Q_SIGNAL void iconChanged(const QImage&);
		Q_SIGNAL void secondaryIconChanged(const QImage&);
		Q_SIGNAL void actionsChanged();
		Q_SIGNAL void hintsChanged(Hints);

		Q_SIGNAL void dismissed(); // NotificationHandler drops that Notification
		Q_SIGNAL void completed(); // NotificationSource deletes the Notification
};

Q_DECLARE_OPERATORS_FOR_FLAGS(Notification::Hints)
Q_FLAGS(Notification::Hints);
} // namespace notifications
} // namespace unity

#endif // PLUGINS_NOTIFICATIONS_NOTIFICATION_H
